from .exporter import Exporter
from .fetcher import Fetcher

import click
import logging

logging.basicConfig(format="%(asctime)s %(message)s", level=logging.INFO)


@click.command()
@click.option(
    '--token', '-t',
    help="API auth-token",
    required=True,
)
@click.option(
    '--url', '-u', '-s',
    help="Netbox URL",
    required=True,
)
@click.option(
    '--dir', '-d', 'dir_',
    help="Destination directory",
    default="netbox",
)
def main(token, url, dir_):
    """Fetch and export you Netbox inventory to a local directory."""

    fetcher = Fetcher(url, token)
    exporter = Exporter(dir_)

    for ctx_prefix, prefix_url in fetcher.fetch_index().items():
        for ctx_suffix, url in fetcher.fetch_index(prefix_url).items():
            ctx = f"{ctx_prefix}/{ctx_suffix}"
            data = fetcher.fetch_data(url)
            exporter.dump(ctx, data)



import logging
import os
import yaml


class Exporter(object):
    """Helper class for dumping data into filesystem."""

    def __init__(self, path: str) -> object:
        """
        :param path: path of root directory to receive dumps
        """
        self.root = path

    def dump(self, ctx: str, data: list) -> None:
        """Dump a Netbox dataset to disk in YAML syntax.
        
        :param ctx:     data's context (section)
        :param data:    payload dataset
        """
        if not os.path.exists(self.root):
            os.mkdir(self.root)
        dirname, basename = ctx.split(os.path.sep)
        parent = os.path.join(self.root, dirname)
        if not os.path.exists(parent):
            os.mkdir(parent)
        path = os.path.join(parent, basename)
        with open(path, "w") as dumpfile:
            dumpfile.write(yaml.dump(data))




import logging
import requests


class FetcherException(Exception):
    pass


class Fetcher(object):
    """Convenience class for fetching data from Netbox."""
    
    LIMIT = 1000
    BLACKLISTED_ENDPOINTS = ['status']

    def __init__(self, url: str, apikey: str) -> object:
        """
        :param url:     the URL to Netbox
        :param apikey:  the API auth-token
        """
        if not url.endswith('/api') and not url.endswith('/api/'):
            url = f"{url}/api/"
        self.url = url
        self.apikey = apikey

    @property
    def headers(self):
        """Helper property for obtaining the preformatted request headers."""
        return {
                "Authorization": f"Token {self.apikey}",
                "Content-Type": "application/json",
        }

    def fetch_index(self, url: str = None) -> dict:
        """Fetch an index from Netbox.

        Ignores all top-level sections listed in self.BLACKLISTED_ENDPOINTS.
        
        :param url: the URL of the index to fetch
        :return:    {section: url, ...}
        """
        if url is None:
            url = self.url
        logging.info(f"Fetching index: {url}")
        req = requests.get(
            url,
            headers=self.headers,
        )
        rc = req.status_code
        if rc != requests.status_codes.codes['OK']:
            raise FetcherException(f"Returned: {rc}")
        return {
            k: v for k, v in req.json().items()
            if k not in self.BLACKLISTED_ENDPOINTS
        }

    def fetch_data(self, url: str) -> list:
        """Fetch data from Netbox.
        
        Will loop through pagination until all data is retrieved.

        :param url: URL to fetch the data from
        :return: [item, ...]
        """
        data = []
        url = f"{url}?limit={self.LIMIT}"
        while url:
            logging.debug(f"Fetching: {url}")
            req = requests.get(
                url,
                headers=self.headers,
            )
            rc = req.status_code
            if rc != requests.status_codes.codes['OK']:
                if rc != 400:
                    raise FetcherException(f"Returned: {rc}")
            res = req.json()
            if 'results' in res:
                data.extend(res['results'])
            url = None
            if 'next' in res:
                if res['next'] != url:
                    url = res['next']
        return data



from setuptools import setup

setup(
    install_requires=[
        'click',
        'netbox-client',
        'requests',
        'pyyaml',
    ],
    extras_require={
        'dev': [
            'build',
            'ipython',
            'prompt-toolkit',
            'pylint',
            'twine',
        ]
    },
)

